import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1Schema.dart';

import 'IDronesController.dart';

class DronesCommandSet extends CommandSet {
  IDronesController _controller;

  DronesCommandSet(IDronesController controller)
      : _controller = controller,
        super() {
    addCommand(_makeGetDronesCommand());
    addCommand(_makeGetDroneByIdCommand());
    addCommand(_makeGetDroneBySerialCommand());
    addCommand(_makeCreateDroneCommand());
    addCommand(_makeUpdateDroneCommand());
    addCommand(_makeDeleteDroneByIdCommand());
  }

  ICommand _makeGetDronesCommand() {
    return Command(
        'get_drones',
        ObjectSchema(true)
            .withOptionalProperty('filter', FilterParamsSchema())
            .withOptionalProperty('paging', PagingParamsSchema()),
        (String? correlationId, Parameters args) {
      var filter = FilterParams.fromValue(args.get('filter'));
      var paging = PagingParams.fromValue(args.get('paging'));
      return _controller.getDrones(correlationId, filter, paging);
    });
  }

  ICommand _makeGetDroneByIdCommand() {
    return Command('get_drone_by_id',
        ObjectSchema(true).withRequiredProperty('drone_id', TypeCode.String),
        (String? correlationId, Parameters args) {
      var droneId = args.getAsString('drone_id');
      return _controller.getDroneById(correlationId, droneId);
    });
  }

  ICommand _makeGetDroneBySerialCommand() {
    return Command('get_drone_by_serial',
        ObjectSchema(true).withRequiredProperty('serial', TypeCode.String),
        (String? correlationId, Parameters args) {
      var serial = args.getAsString('serial');
      return _controller.getDroneBySerial(correlationId, serial);
    });
  }

  ICommand _makeCreateDroneCommand() {
    return Command('create_drone',
        ObjectSchema(true).withRequiredProperty('drone', DroneV1Schema()),
        (String? correlationId, Parameters args) {
      var drone = DroneV1();
      drone.fromJson(args.get('drone'));
      return _controller.createDrone(correlationId, drone);
    });
  }

  ICommand _makeUpdateDroneCommand() {
    return Command('update_drone',
        ObjectSchema(true).withRequiredProperty('drone', DroneV1Schema()),
        (String? correlationId, Parameters args) {
      var drone = DroneV1();
      drone.fromJson(args.get('drone'));
      return _controller.updateDrone(correlationId, drone);
    });
  }

  ICommand _makeDeleteDroneByIdCommand() {
    return Command('delete_drone_by_id',
        ObjectSchema(true).withRequiredProperty('drone_id', TypeCode.String),
        (String? correlationId, Parameters args) {
      var droneId = args.getAsString('drone_id');
      return _controller.deleteDroneById(correlationId, droneId);
    });
  }
}
