import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1.dart';
import 'package:service_drones_dart/src/logic/IDronesController.dart';
import 'package:service_drones_dart/src/persistence/IDronesPersistence.dart';

import 'DronesCommandSet.dart';

class DronesController
    implements IDronesController, IConfigurable, IReferenceable, ICommandable {
  IDronesPersistence? persistence;
  DronesCommandSet? commandSet;

  @override
  void configure(ConfigParams config) {}

  @override
  void setReferences(IReferences references) {
    persistence = references.getOneRequired<IDronesPersistence>(Descriptor(
        'eic-stopfires-services-drones', 'persistence', '*', '*', '1.0'));
  }

  @override
  CommandSet getCommandSet() {
    commandSet ??= DronesCommandSet(this);
    return commandSet!;
  }

  @override
  Future<DataPage<DroneV1?>> getDrones(
      String? correlationId, FilterParams filter, PagingParams paging) {
    return persistence!.getPageByFilter(correlationId, filter, paging);
  }

  @override
  Future<DroneV1?> getDroneById(String? correlationId, String droneId) {
    return persistence!.getOneById(correlationId, droneId);
  }

  @override
  Future<DroneV1?> getDroneBySerial(String? correlationId, String serial) {
    return persistence!.getOneBySerial(correlationId, serial);
  }

  @override
  Future<DroneV1?> createDrone(String? correlationId, DroneV1 drone) {
    drone.id = drone.id ?? IdGenerator.nextLong();
    return persistence!.create(correlationId, drone);
  }

  @override
  Future<DroneV1?> updateDrone(String? correlationId, DroneV1 drone) {
    return persistence!.update(correlationId, drone);
  }

  @override
  Future<DroneV1?> deleteDroneById(String? correlationId, String droneId) {
    return persistence!.deleteById(correlationId, droneId);
  }
}
