import 'package:pip_services3_commons/src/data/PagingParams.dart';
import 'package:pip_services3_commons/src/data/FilterParams.dart';
import 'package:pip_services3_commons/src/data/DataPage.dart';
import 'package:pip_services3_data/pip_services3_data.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1.dart';
import 'package:service_drones_dart/src/persistence/IDronesPersistence.dart';

class DronesMemoryPersistence
    extends IdentifiableMemoryPersistence<DroneV1, String>
    implements IDronesPersistence {
  Function _composeFilter(FilterParams? filter) {
    filter = filter ?? FilterParams();

    var id = filter.getAsNullableString('id');
    var type = filter.getAsNullableString('type');
    var ownerId = filter.getAsNullableString('owner_id');
    var serial = filter.getAsNullableString('serial');
    var range = filter.getAsNullableInteger('range');
    var ownerIds = filter.getAsObject('owner_ids');

    if (ownerIds != null && ownerIds is String) {
      ownerIds = ownerIds.split(',');
    }
    if (ownerIds != null && !(ownerIds is List)) {
      ownerIds = null;
    }

    return (DroneV1 item) {
      if (id != null && item.id != id) {
        return false;
      }
      if (type != null && item.type != type) {
        return false;
      }
      if (ownerId != null && item.ownerId != ownerId) {
        return false;
      }

      if (serial != null && item.serial != serial) {
        return false;
      }

      if (range != null && item.range != range) {
        return false;
      }

      if (ownerIds != null && !(ownerIds as List).contains(item.ownerId)) {
        return false;
      }
      return true;
    };
  }

  @override
  Future<DroneV1?> getOneBySerial(String? correlationId, String serial) async {
    var result = items.where((item) => item.serial == serial).toList();
    var item = result.isNotEmpty ? result[0] : null;

    if (item != null) {
      logger.trace(correlationId, 'Found drone by %s', [serial]);
    } else {
      logger.trace(correlationId, 'Cannot find drone by %s', [serial]);
    }

    return item;
  }

  @override
  Future<DataPage<DroneV1?>> getPageByFilter(
      String? correlationId, FilterParams filter, PagingParams paging) {
    return super
        .getPageByFilterEx(correlationId, _composeFilter(filter), paging, null);
  }
}
