import 'package:mongo_dart_query/mongo_dart_query.dart' as mngquery;

import 'package:pip_services3_commons/src/data/PagingParams.dart';
import 'package:pip_services3_commons/src/data/FilterParams.dart';
import 'package:pip_services3_commons/src/data/DataPage.dart';
import 'package:pip_services3_mongodb/pip_services3_mongodb.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1.dart';
import 'package:service_drones_dart/src/persistence/IDronesPersistence.dart';

class DronesMongoDbPersistence
    extends IdentifiableMongoDbPersistence<DroneV1, String>
    implements IDronesPersistence {
  DronesMongoDbPersistence() : super('drones') {
    maxPageSize = 1000;
  }
  Map<String, dynamic>? _composeFilter(FilterParams? filter) {
    filter = filter ?? FilterParams();

    var criteria = [];

    var id = filter.getAsNullableString('id');
    if (id != null) {
      criteria.add({'_id': id});
    }

    var type = filter.getAsNullableString('type');
    if (type != null) {
      criteria.add({'type': type});
    }

    var ownerId = filter.getAsNullableString('owner_id');
    if (ownerId != null) {
      criteria.add({'owner_id': ownerId});
    }

    var serial = filter.getAsNullableString('serial');
    if (serial != null) {
      criteria.add({'serial': serial});
    }

    var range = filter.getAsNullableString('range');
    if (range != null) {
      criteria.add({'range': serial});
    }

    var ownerIds = filter.getAsObject('owner_ids');
    if (ownerIds is String) {
      ownerIds = ownerIds.split(',');
    }
    if (ownerIds is List) {
      criteria.add({
        'owner_id': {r'$in': ownerIds}
      });
    }

    return criteria.isNotEmpty ? {r'$and': criteria} : null;
  }

  @override
  Future<DroneV1?> getOneBySerial(String? correlationId, String serial) async {
    var filter = {'serial': serial};
    var query = mngquery.SelectorBuilder();
    var selector = <String, dynamic>{};
    if (serial.isNotEmpty) {
      selector[r'$query'] = filter;
    }
    query.raw(selector);

    var item = await collection!.findOne(filter);

    if (item == null) {
      logger.trace(correlationId, 'Nothing found from %s with serial = %s',
          [collectionName, serial]);
      return null;
    }
    logger.trace(correlationId, 'Retrieved from %s with serial = %s',
        [collectionName, serial]);
    return convertToPublic(item);
  }

  @override
  Future<DataPage<DroneV1?>> getPageByFilter(
      String? correlationId, FilterParams filter, PagingParams paging) {
    return super
        .getPageByFilterEx(correlationId, _composeFilter(filter), paging, null);
  }
}
