import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:service_drones_dart/src/data/version1/data.dart';
import 'package:service_drones_dart/src/logic/logic.dart';
import 'package:service_drones_dart/src/persistence/persistence.dart';
import 'package:test/test.dart';

final DRONE1 = DroneV1(
    id: '1',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0001',
    serial: '123',
    range: 100,
    base: LocationV1(lat: 50, long: 70));

final DRONE2 = DroneV1(
    id: '2',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0002',
    serial: '456',
    range: 200,
    base: LocationV1(lat: 30, long: 40));

void main() {
  group('DronesController', () {
    late DronesMemoryPersistence persistence;
    late DronesController controller;

    setUp(() async {
      persistence = DronesMemoryPersistence();
      persistence.configure(ConfigParams());

      controller = DronesController();
      controller.configure(ConfigParams());

      var references = References.fromTuples([
        Descriptor('eic-stopfires-services-drones', 'persistence', 'memory',
            'default', '1.0'),
        persistence,
        Descriptor('eic-stopfires-services-drones', 'controller', 'default',
            'default', '1.0'),
        controller
      ]);

      controller.setReferences(references);

      await persistence.open(null);
    });

    tearDown(() async {
      await persistence.close(null);
    });

    test('CRUD Operations', () async {
      DroneV1 drone1;

      // Create the first drone
      var drone = await persistence.create(null, DRONE1);

      expect(drone, isNotNull);
      expect(DRONE1.type, drone!.type);
      expect(DRONE1.ownerId, drone.ownerId);
      expect(DRONE1.serial, drone.serial);
      expect(DRONE1.range, drone.range);
      expect(DRONE1.base, isNotNull);

      // Create the second drone
      drone = await persistence.create(null, DRONE2);

      expect(drone, isNotNull);
      expect(DRONE2.type, drone!.type);
      expect(DRONE2.ownerId, drone.ownerId);
      expect(DRONE2.serial, drone.serial);
      expect(DRONE2.range, drone.range);
      expect(DRONE2.base, isNotNull);

      // Get all beacons
      var page = await persistence.getPageByFilter(
          null, FilterParams(), PagingParams());
      expect(page, isNotNull);
      expect(page.data.length, 2);

      drone1 = page.data[0]!;

      // Update the drone
      drone1.serial = 'ABC';

      drone = await persistence.update(null, drone1);
      expect(drone, isNotNull);
      expect(drone1.id, drone!.id);
      expect('ABC', drone.serial);

      // Get drone by udi
      drone = await persistence.getOneBySerial(null, drone1.serial!);
      expect(drone, isNotNull);
      expect(drone1.id, drone!.id);

      // Delete the drone
      drone = await persistence.deleteById(null, drone1.id!);
      expect(drone, isNotNull);
      expect(drone1.id, drone!.id);

      // Try to get deleted drone
      drone = await persistence.getOneById(null, drone1.id!);
      expect(drone, isNull);
    });
  });
}
