import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:service_drones_dart/src/data/version1/DroneTypeV1.dart';
import 'package:service_drones_dart/src/data/version1/DroneV1.dart';
import 'package:service_drones_dart/src/data/version1/LocationV1.dart';
import 'package:service_drones_dart/src/persistence/IDronesPersistence.dart';
import 'package:test/test.dart';

final DRONE1 = DroneV1(
    id: '1',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0001',
    serial: '123',
    range: 100,
    base: LocationV1(lat: 50, long: 70));

final DRONE2 = DroneV1(
    id: '2',
    type: DroneTypeV1.MiniDrone,
    ownerId: '0002',
    serial: '456',
    range: 200,
    base: LocationV1(lat: 30, long: 40));

final DRONE3 = DroneV1(
    id: '3',
    type: DroneTypeV1.Plane,
    ownerId: '0003',
    serial: '789',
    range: 1000,
    base: LocationV1(lat: 10, long: 20));

class DronesPersistenceFixture {
  late IDronesPersistence _persistence;

  DronesPersistenceFixture(IDronesPersistence persistence) {
    expect(persistence, isNotNull);
    _persistence = persistence;
  }

  Future _testCreateDrones() async {
    // Create the first drone
    var drone = await _persistence.create(null, DRONE1);

    expect(drone, isNotNull);
    expect(DRONE1.type, drone!.type);
    expect(DRONE1.ownerId, drone.ownerId);
    expect(DRONE1.serial, drone.serial);
    expect(DRONE1.range, drone.range);
    expect(DRONE1.base, isNotNull);

    // Create the second drone
    drone = await _persistence.create(null, DRONE2);

    expect(drone, isNotNull);
    expect(DRONE2.type, drone!.type);
    expect(DRONE2.ownerId, drone.ownerId);
    expect(DRONE2.serial, drone.serial);
    expect(DRONE2.range, drone.range);
    expect(DRONE2.base, isNotNull);

    // Create the third drone
    drone = await _persistence.create(null, DRONE3);

    expect(drone, isNotNull);
    expect(DRONE3.type, drone!.type);
    expect(DRONE3.ownerId, drone.ownerId);
    expect(DRONE3.serial, drone.serial);
    expect(DRONE3.range, drone.range);
    expect(DRONE3.base, isNotNull);
  }

  Future testCrudOperations() async {
    DroneV1 drone1;

    // Create items
    await _testCreateDrones();

    // Get all beacons
    var page = await _persistence.getPageByFilter(
        null, FilterParams(), PagingParams());
    expect(page, isNotNull);
    expect(page.data.length, 3);

    drone1 = page.data[0]!;

    // Update the drone
    drone1.serial = 'ABC';

    var drone = await _persistence.update(null, drone1);
    expect(drone, isNotNull);
    expect(drone1.id, drone!.id);
    expect('ABC', drone.serial);

    // Get drone by udi
    drone = await _persistence.getOneBySerial(null, drone1.serial!);
    expect(drone, isNotNull);
    expect(drone1.id, drone!.id);

    // Delete the drone
    drone = await _persistence.deleteById(null, drone1.id!);
    expect(drone, isNotNull);
    expect(drone1.id, drone!.id);

    // Try to get deleted drone
    drone = await _persistence.getOneById(null, drone1.id!);
    expect(drone, isNull);
  }

  Future testGetWithFilters() async {
    // Create items

    await _testCreateDrones();

    // Filter by id
    var page = await _persistence.getPageByFilter(
        null, FilterParams.fromTuples(['id', '1']), PagingParams());
    expect(page.data.length, 1);

    // Filter by owner_id
    page = await _persistence.getPageByFilter(
        null, FilterParams.fromTuples(['owner_id', '0002']), PagingParams());
    expect(page.data.length, 1);

    // Filter by owner_ids
    page = await _persistence.getPageByFilter(null,
        FilterParams.fromTuples(['owner_ids', '0001,0003']), PagingParams());
    expect(page.data.length, 2);

    // Filter by site_id
    page = await _persistence.getPageByFilter(
        null,
        FilterParams.fromTuples(['type', DroneTypeV1.MiniDrone]),
        PagingParams());
    expect(page.data.length, 2);
  }
}
